@@ -151,6 +151,15 @@ tkigraph <- function() {
         command=function() {
           .tkigraph.degree("total")
         })
+  tkadd(centrality.menu, "command", label="Plot log-log degree distribution",
+        command=function() {
+          .tkigraph.degree.dist(power=FALSE)
+        })
+  tkadd(centrality.menu, "command", label="Fit a power-law to degree distribution",
+        command=function() {
+          .tkigraph.degree.dist(power=TRUE)
+        })
+  tkadd(centrality.menu, "separator")
   tkadd(centrality.menu, "command", label="Closeness", command=function() {
     .tkigraph.closeness()
   })
@@ -509,7 +518,10 @@ tkigraph <- function() {
   cn <- rep("", ncol(tab))
   if (ncol(tab)>=3) { cn[3] <- "weight" }
   colnames(tab) <- cn
-  g <- graph.data.frame(tab, directed=TRUE)
+  read <- .tkigraph.dialogbox(TITLE="Importing an edge list",
+                              directed=list(name="Directed", type="boolean",
+                                default="FALSE"))
+  g <- graph.data.frame(tab, directed=read$directed)
   g <- set.graph.attribute(g, "name", "Imported edge list")
   .tkigraph.add.graph(g)
 }
@@ -1136,6 +1148,37 @@ tkigraph <- function() {
                      showmean=mv)
 }
 
+.tkigraph.degree.dist <- function(power=FALSE) {
+  gnos <- .tkigraph.get.selected()
+  if (length(gnos)!=1) {
+    .tkigraph.error("Please select exactly one graph")
+    return()
+  }
+  graphs <- get("graphs", .tkigraph.env)
+  read <- .tkigraph.dialogbox(TITLE="Choose degree type",
+                              type=list(name="Degree type",
+                                type="listbox", default="0",
+                                values=c("Out", "In", "Total")))
+  mode <- c("out", "in", "all")[read$type+1]
+  deg <- degree(graphs[[gnos]], mode=mode)
+  x11()
+  h <- hist(deg, -1:max(deg), plot=FALSE)$density
+  plot(0:max(deg), h, xlab="Degree", ylab="Relative frequency",
+       type="b", main="Degree distribution", log="xy")
+
+  if (power) {
+    if (max(deg)<10) {
+      .tkigraph.error("Degrees are too small for a power-law fit")
+      return()
+    }
+    fit <- power.law.fit(deg, xmin=10)
+    lines(0:max(deg), (0:max(deg))^(-coef(fit)), col="red")
+    legend("topright", c(paste("exponent:", round(coef(fit), 2)),
+                         paste("standard error:", round(sqrt(vcov(fit)), 2))),
+           bty="n", cex=1.5)
+  }
+}
+
 .tkigraph.closeness <- function() {
   gnos <- .tkigraph.get.selected()
   if (length(gnos)!=1) {
